@@ -1,6 +1,34 @@
 ## function fits Pagel '94 model of correlated evolution of two binary characters
 ## uses fitMk, ape::ace, or geiger::fitDiscrete internally
-## written by Liam J. Revell 2014, 2015, 2016, 2020
+## written by Liam J. Revell 2014, 2015, 2016, 2020, 2022, 2023
+
+anova.fitPagel<-function(object,...){
+	fits<-list(...)
+	nm<-c(
+		"independent",
+		deparse(substitute(object)),
+		if(length(fits)>0) 
+		sapply(substitute(list(...))[-1],deparse)
+	)
+	logL<-c(object$independent.logL,
+		object$dependent.logL,
+		if(length(fits)>0)
+		sapply(fits,function(x) x$dependent.logL))
+	df<-c(attr(object$independent.logL,"df"),
+		attr(object$dependent.logL,"df"),
+		if(length(fits)>0) 
+		sapply(fits,function(x) attr(x$dependent.logL,"df")))
+	AICvals<-c(object$independent.AIC,
+		object$dependent.AIC,
+		if(length(fits)>0) 
+		sapply(fits,function(x) x$dependent.AIC))
+	ww<-aic.w(AICvals)
+	result<-data.frame(logL,df,AICvals,unclass(ww))
+	rownames(result)<-nm
+	colnames(result)<-c("log(L)","d.f.","AIC","weight")
+	print(result)
+	invisible(result)
+}
 
 fitPagel<-function(tree,x,y,method="fitMk",model="ARD",dep.var="xy",...){
 	if(!inherits(tree,"phylo")) stop("tree should be object of class \"phylo\".")
@@ -184,8 +212,12 @@ make.sym<-function(X){
 }
 
 ## S3 plot method for objects of class "fitPagel
-## written by Liam J. Revell 2016
+## written by Liam J. Revell 2016, 2023
 plot.fitPagel<-function(x,...){
+	if(hasArg(mar)) mar<-list(...)$mar
+	else mar<-c(1.1,2.1,3.1,2.1)
+	if(hasArg(show)) show<-list(...)$show
+	else show<-"both"
 	if(hasArg(signif)) signif<-list(...)$signif
 	else signif<-3
 	if(hasArg(main)) main<-list(...)$main
@@ -213,117 +245,121 @@ plot.fitPagel<-function(x,...){
 		LWD.dep[LWD.dep>max.lwd]<-max.lwd
 	} else LWD.ind<-LWD.dep<-matrix(lwd,nrow(x$dependent.Q),
 		ncol(x$dependent.Q))
-	par(mfrow=c(2,1))
-	## INDEPENDENT MODEL
-	plot.new()
-	par(mar=c(1.1,2.1,3.1,2.1))
-	plot.window(xlim=c(0,2),ylim=c(0,1),asp=1)
-	mtext(if(!is.null(main)) main[1] else "a) Independent model",
-		side=3,adj=0,line=1.2,cex=cex.main)
-	## trait 1
-	text(x=0.15,y=1,"Trait 1:",cex=cex.sub)
-	arrows(x0=0.5,y0=0.15,x1=0.5,y1=0.85,
-		lwd=max(LWD.ind[3,1],1),
-		lty=if(LWD.ind[3,1]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=0.55,y0=0.85,x1=0.55,y1=0.15,
-		lwd=max(LWD.ind[1,3],1),
-		lty=if(LWD.ind[1,3]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	text(x=0.525,y=0.95,namesplit(rownames(x$dependent.Q)[1])[1],
-		cex=cex.traits)
-	text(x=0.525,y=0.05,namesplit(rownames(x$dependent.Q)[3])[1],
-		cex=cex.traits)
-	text(x=0.60,y=0.5,round(x$independent.Q[1,3],signif),cex=cex.rates,srt=90)
-	text(x=0.45,y=0.5,round(x$independent.Q[3,1],signif),cex=cex.rates,srt=90)
-	## trait 2
-	text(x=1.3,y=1,"Trait 2:",cex=cex.sub)	
-	arrows(x0=1.65,y0=0.15,x1=1.65,y1=0.85,
-		lwd=max(LWD.ind[2,1],1),
-		lty=if(LWD.ind[2,1]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=1.70,y0=0.85,x1=1.70,y1=0.15,
-		lwd=max(LWD.ind[1,2],1),
-		lty=if(LWD.ind[1,2]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	text(x=1.675,y=0.95,namesplit(rownames(x$dependent.Q)[1])[2],
-		cex=cex.traits)
-	text(x=1.675,y=0.05,namesplit(rownames(x$dependent.Q)[2])[2],
-		cex=cex.traits)
-	text(x=1.75,y=0.5,round(x$independent.Q[1,2],signif),cex=cex.rates,srt=90)
-	text(x=1.60,y=0.5,round(x$independent.Q[2,1],signif),cex=cex.rates,srt=90)
-	## DEPENDENT MODEL
-	collapse<-
-		if(any(sapply(strsplit(rownames(x$dependent.Q),""),length)>6)) 
-		",\n" else ", "
-	plot.new()
-	par(mar=c(1.1,2.1,3.1,2.1))
-	plot.window(xlim=c(0,2),ylim=c(0,1),asp=1)
-	mtext(if(!is.null(main)) main[2] else "b) Dependent model",
-		side=3,adj=0,line=1.2,cex=cex.main)
-	text(x=0.15,y=0.95,"Trait 1,\nTrait 2:",cex=cex.sub)
-	arrows(x0=0.5,y0=0.15,x1=0.5,y1=0.85,
-		lwd=max(LWD.dep[3,1],1),
-		lty=if(LWD.dep[3,1]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=0.55,y0=0.85,x1=0.55,y1=0.15,
-		lwd=max(LWD.dep[1,3],1),
-		lty=if(LWD.dep[1,3]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=1.45,y0=0.05,x1=0.75,y1=0.05,
-		lwd=max(LWD.dep[4,3],1),
-		lty=if(LWD.dep[4,3]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=0.75,y0=0.1,x1=1.45,y1=0.1,
-		lwd=max(LWD.dep[3,4],1),
-		lty=if(LWD.dep[3,4]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=1.65,y0=0.15,x1=1.65,y1=0.85,
-		lwd=max(LWD.dep[4,2],1),
-		lty=if(LWD.dep[4,2]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=1.7,y0=0.85,x1=1.7,y1=0.15,
-		lwd=max(LWD.dep[2,4],1),
-		lty=if(LWD.dep[2,4]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=1.45,y0=0.9,x1=0.75,y1=0.9,
-		lwd=max(LWD.dep[2,1],1),
-		lty=if(LWD.dep[2,1]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	arrows(x0=0.75,y0=0.95,x1=1.45,y1=0.95,
-		lwd=max(LWD.dep[1,2],1),
-		lty=if(LWD.dep[1,2]==0) "dashed" else "solid",
-		length=0.15,lend=3,angle=20)
-	## add states
-	text(x=0.525,y=0.95,
-		paste(namesplit(rownames(x$dependent.Q)[1]),
-		collapse=collapse),cex=cex.traits)
-	text(x=1.675,y=0.95,
-		paste(namesplit(rownames(x$dependent.Q)[2]),
-		collapse=collapse),cex=cex.traits)
-	text(x=1.675,y=0.05,
-		paste(namesplit(rownames(x$dependent.Q)[4]),
-		collapse=collapse),cex=cex.traits)
-	text(x=0.525,y=0.05,
-		paste(namesplit(rownames(x$dependent.Q)[3]),
-		collapse=collapse),cex=cex.traits)
-	## add rates
-	text(x=1.1,y=1,round(x$dependent.Q[1,2],signif),
-		cex=cex.rates)
-	text(x=1.1,y=0.85,round(x$dependent.Q[2,1],signif),
-		cex=cex.rates)
-	text(x=1.6,y=0.5,round(x$dependent.Q[4,2],signif),
-		cex=cex.rates,srt=90)
-	text(x=1.75,y=0.5,round(x$dependent.Q[2,4],signif),
-		cex=cex.rates,srt=90)
-	text(x=1.1,y=0,round(x$dependent.Q[4,3],signif),
-		cex=cex.rates)
-	text(x=1.1,y=0.15,round(x$dependent.Q[3,4],signif),
-		cex=cex.rates)
-	text(x=0.45,y=0.5,round(x$dependent.Q[3,1],signif),
-		cex=cex.rates,srt=90)
-	text(x=0.6,y=0.5,round(x$dependent.Q[1,3],signif),
-		cex=cex.rates,srt=90)
+	if(show=="both") par(mfrow=c(2,1))
+	if(show%in%c("both","independent")){
+		## INDEPENDENT MODEL
+		plot.new()
+		par(mar=mar)
+		plot.window(xlim=c(0,2),ylim=c(0,1),asp=1)
+		mtext(if(!is.null(main)) main[1] else "a) Independent model",
+			side=3,adj=0,line=1.2,cex=cex.main)
+		## trait 1
+		text(x=0.15,y=1,"Trait 1:",cex=cex.sub)
+		arrows(x0=0.5,y0=0.15,x1=0.5,y1=0.85,
+			lwd=max(LWD.ind[3,1],1),
+			lty=if(LWD.ind[3,1]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=0.55,y0=0.85,x1=0.55,y1=0.15,
+			lwd=max(LWD.ind[1,3],1),
+			lty=if(LWD.ind[1,3]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		text(x=0.525,y=0.95,namesplit(rownames(x$dependent.Q)[1])[1],
+			cex=cex.traits)
+		text(x=0.525,y=0.05,namesplit(rownames(x$dependent.Q)[3])[1],
+			cex=cex.traits)
+		text(x=0.60,y=0.5,round(x$independent.Q[1,3],signif),cex=cex.rates,srt=90)
+		text(x=0.45,y=0.5,round(x$independent.Q[3,1],signif),cex=cex.rates,srt=90)
+		## trait 2
+		text(x=1.3,y=1,"Trait 2:",cex=cex.sub)	
+		arrows(x0=1.65,y0=0.15,x1=1.65,y1=0.85,
+			lwd=max(LWD.ind[2,1],1),
+			lty=if(LWD.ind[2,1]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=1.70,y0=0.85,x1=1.70,y1=0.15,
+			lwd=max(LWD.ind[1,2],1),
+			lty=if(LWD.ind[1,2]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		text(x=1.675,y=0.95,namesplit(rownames(x$dependent.Q)[1])[2],
+			cex=cex.traits)
+		text(x=1.675,y=0.05,namesplit(rownames(x$dependent.Q)[2])[2],
+			cex=cex.traits)
+		text(x=1.75,y=0.5,round(x$independent.Q[1,2],signif),cex=cex.rates,srt=90)
+		text(x=1.60,y=0.5,round(x$independent.Q[2,1],signif),cex=cex.rates,srt=90)
+	}
+	if(show%in%c("both","dependent")){
+		## DEPENDENT MODEL
+		collapse<-
+			if(any(sapply(strsplit(rownames(x$dependent.Q),""),length)>6)) 
+			",\n" else ", "
+		plot.new()
+		par(mar=mar)
+		plot.window(xlim=c(0,2),ylim=c(0,1),asp=1)
+		mtext(if(!is.null(main)) main[2] else "b) Dependent model",
+			side=3,adj=0,line=1.2,cex=cex.main)
+		text(x=0.15,y=0.95,"Trait 1,\nTrait 2:",cex=cex.sub)
+		arrows(x0=0.5,y0=0.15,x1=0.5,y1=0.85,
+			lwd=max(LWD.dep[3,1],1),
+			lty=if(LWD.dep[3,1]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=0.55,y0=0.85,x1=0.55,y1=0.15,
+			lwd=max(LWD.dep[1,3],1),
+			lty=if(LWD.dep[1,3]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=1.45,y0=0.05,x1=0.75,y1=0.05,
+			lwd=max(LWD.dep[4,3],1),
+			lty=if(LWD.dep[4,3]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=0.75,y0=0.1,x1=1.45,y1=0.1,
+			lwd=max(LWD.dep[3,4],1),
+			lty=if(LWD.dep[3,4]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=1.65,y0=0.15,x1=1.65,y1=0.85,
+			lwd=max(LWD.dep[4,2],1),
+			lty=if(LWD.dep[4,2]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=1.7,y0=0.85,x1=1.7,y1=0.15,
+			lwd=max(LWD.dep[2,4],1),
+			lty=if(LWD.dep[2,4]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=1.45,y0=0.9,x1=0.75,y1=0.9,
+			lwd=max(LWD.dep[2,1],1),
+			lty=if(LWD.dep[2,1]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		arrows(x0=0.75,y0=0.95,x1=1.45,y1=0.95,
+			lwd=max(LWD.dep[1,2],1),
+			lty=if(LWD.dep[1,2]==0) "dashed" else "solid",
+			length=0.15,lend=3,angle=20)
+		## add states
+		text(x=0.525,y=0.95,
+			paste(namesplit(rownames(x$dependent.Q)[1]),
+			collapse=collapse),cex=cex.traits)
+		text(x=1.675,y=0.95,
+			paste(namesplit(rownames(x$dependent.Q)[2]),
+			collapse=collapse),cex=cex.traits)
+		text(x=1.675,y=0.05,
+			paste(namesplit(rownames(x$dependent.Q)[4]),
+			collapse=collapse),cex=cex.traits)
+		text(x=0.525,y=0.05,
+			paste(namesplit(rownames(x$dependent.Q)[3]),
+			collapse=collapse),cex=cex.traits)
+		## add rates
+		text(x=1.1,y=1,round(x$dependent.Q[1,2],signif),
+			cex=cex.rates)
+		text(x=1.1,y=0.85,round(x$dependent.Q[2,1],signif),
+			cex=cex.rates)
+		text(x=1.6,y=0.5,round(x$dependent.Q[4,2],signif),
+			cex=cex.rates,srt=90)
+		text(x=1.75,y=0.5,round(x$dependent.Q[2,4],signif),
+			cex=cex.rates,srt=90)
+		text(x=1.1,y=0,round(x$dependent.Q[4,3],signif),
+			cex=cex.rates)
+		text(x=1.1,y=0.15,round(x$dependent.Q[3,4],signif),
+			cex=cex.rates)
+		text(x=0.45,y=0.5,round(x$dependent.Q[3,1],signif),
+			cex=cex.rates,srt=90)
+		text(x=0.6,y=0.5,round(x$dependent.Q[1,3],signif),
+			cex=cex.rates,srt=90)
+	}
 }
 
 namesplit<-function(x){
