@@ -1,5 +1,5 @@
 ## function depends on phytools (& dependencies) and maps (& dependencies)
-## written by Liam J. Revell 2013, 2017, 2019, 2022
+## written by Liam J. Revell 2013, 2017, 2019, 2022, 2023
 
 phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
@@ -14,6 +14,8 @@ phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 	else xlim<-c(-180,180)
 	if(hasArg(ylim)) ylim<-list(...)$ylim
 	else ylim<-c(-90,90)
+	if(hasArg(quiet)) quiet<-list(...)$quiet
+	else quiet<-FALSE
 	# create a map
 	map<-map(database,regions,xlim=xlim,ylim=ylim,plot=FALSE,fill=TRUE,resolution=0)
 	# if rotate
@@ -23,7 +25,7 @@ phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 	if(rotate&&type=="phylogram"){
 		cc<-aggregate(coords,by=list(rownames(coords)),mean)
 		cc<-matrix(as.matrix(cc[,2:3]),nrow(cc),2,dimnames=list(cc[,1],colnames(cc)[2:3]))
-		tree<-minRotate(tree,cc[,if(direction=="rightwards") 1 else 2])
+		tree<-minRotate(tree,cc[,if(direction=="rightwards") 1 else 2],print=!quiet)
 	} else direction<-"unoptimized"
 	x<-list(tree=tree,map=map,coords=coords,direction=direction)
 	class(x)<-"phylo.to.map"
@@ -32,7 +34,7 @@ phylo.to.map<-function(tree,coords,rotate=TRUE,...){
 }
 
 ## S3 method to plot object of class "phylo.to.map"
-## written by Liam J. Revell 2013, 2014, 2016, 2019, 2020
+## written by Liam J. Revell 2013, 2014, 2016, 2019, 2020, 2022
 
 plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	type<-type[1]
@@ -91,6 +93,8 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	else pts<-TRUE
 	if(hasArg(col.edge)) col.edge<-list(...)$col.edge
 	else col.edge<-rep(par()$fg,nrow(x$tree$edge))
+	if(hasArg(map.bg)) map.bg<-list(...)$map.bg
+	else map.bg<-"gray95"
 	if(type=="phylogram"){
 		if(x$direction=="downwards"&&direction=="rightwards"){
 			cat("\"phylo.to.map\" direction is \"downwards\" but plot direction has been given as \"rightwards\".\n")
@@ -122,7 +126,7 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 	plot.new()
 	par(mar=mar)
 	plot.window(xlim=xlim,ylim=ylim,asp=asp)
-	map(map,add=TRUE,fill=TRUE,col="gray95",mar=rep(0,4))
+	map(map,add=TRUE,fill=TRUE,col=map.bg,mar=rep(0,4))
 	if(type=="phylogram"){
 		## preliminaries
 		cw<-reorder(tree,"cladewise")
@@ -132,7 +136,7 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			# plot a white rectangle
 			dx<-abs(diff(xlim))
 			rect(xlim[1]-1.04*dx,ylim[2]-split[1]*(ylim[2]-ylim[1]),
-				xlim[2]+1.04*dx,ylim[2],col="white",border="white")
+				xlim[2]+1.04*dx,ylim[2],col=par()$bg,border=par()$bg)
 			# rescale tree so it fits in the upper half of the plot
 			# with enough space for labels
 			pdin<-par()$din[2]
@@ -184,14 +188,14 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 				font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,
 				label.offset=fsize*strwidth(" ")/(par()$usr[2]-
 				par()$usr[1])*(par()$usr[4]-par()$usr[3]),
-				x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
-				direction=direction,tip.color="black",Ntip=Ntip(cw),
+				x.lim=xlim,y.lim=ylim,
+				direction=direction,tip.color=par()$fg,Ntip=Ntip(cw),
 				Nnode=cw$Nnode,edge=cw$edge,xx=x,yy=sapply(1:(Ntip(cw)+
 				cw$Nnode),function(x,y,z) y[match(x,z)],y=Y,z=cw$edge))
 		} else {
 			dy<-abs(diff(ylim))
 			rect(xlim[1],ylim[1],xlim[1]+split[1]*(xlim[2]-
-				xlim[1]),ylim[2],col="white",border="white")
+				xlim[1]),ylim[2],col=par()$bg,border=par()$bg)
 			sh<-fsize*strwidth(paste(" ",cw$tip.label,sep=""))+
 				0.2*fsize*strwidth("W")
 			cw$edge.length<-cw$edge.length/max(nodeHeights(cw))*
@@ -228,8 +232,8 @@ plot.phylo.to.map<-function(x,type=c("phylogram","direct"),...){
 			PP<-list(type="phylogram",use.edge.length=TRUE,node.pos=1,
 				show.tip.label=if(ftype) TRUE else FALSE,show.node.label=FALSE,
 				font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=0.1,
-				x.lim=par()$usr[1:2],y.lim=par()$usr[3:4],
-				direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
+				x.lim=xlim,y.lim=ylim,
+				direction=direction,tip.color=par()$fg,Ntip=Ntip(cw),Nnode=cw$Nnode,
 				edge=cw$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
 				function(x,y,z) y[match(x,z)],y=X,z=cw$edge),yy=y)
 		}
