@@ -1,17 +1,17 @@
 ## functions plot stochastic character mapped trees
-## written by Liam Revell 2011-2021
+## written by Liam Revell 2011-2023
 
 plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 	pts=FALSE,node.numbers=FALSE,mar=NULL,add=FALSE,offset=NULL,direction="rightwards",
-	type="phylogram",setEnv=TRUE,part=1.0,xlim=NULL,ylim=NULL,nodes="intermediate",
-	tips=NULL,maxY=NULL,hold=TRUE,split.vertical=FALSE,lend=2,asp=NA,outline=FALSE,
-	plot=TRUE){
+	type="phylogram",setEnv=TRUE,part=if(type=="arc") 0.5 else 1.0,xlim=NULL,ylim=NULL,
+	nodes="intermediate",tips=NULL,maxY=NULL,hold=TRUE,split.vertical=FALSE,lend=2,asp=NA,
+	outline=FALSE,plot=TRUE,underscore=FALSE,arc_height=2){
 	if(inherits(tree,"multiPhylo")){
 		par(ask=TRUE)
 		for(i in 1:length(tree)) plotSimmap(tree[[i]],colors=colors,fsize=fsize,
 			ftype=ftype,lwd=lwd,pts=pts,node.numbers=node.numbers,mar,add,offset,
 			direction,type,setEnv,part,xlim,ylim,nodes,tips,maxY,hold,split.vertical,
-			lend,asp,outline,plot)
+			lend,asp,outline,plot,underscore)
 	} else {
 		# check tree
 		if(!inherits(tree,"phylo")) stop("tree should be object of class \"phylo\"")
@@ -30,7 +30,7 @@ plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 			}
 		}
 		# swap out "_" character for spaces (assumes _ is a place holder)
-		tree$tip.label<-gsub("_"," ",tree$tip.label)
+		if(!underscore) tree$tip.label<-gsub("_"," ",tree$tip.label)
 		# get margin
 		if(is.null(mar)) mar=rep(0.1,4)
 		if(hold) null<-dev.hold()
@@ -43,12 +43,12 @@ plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 					black[]<-fg
 					updownPhylogram(tree,colors=black,fsize,ftype,lwd=lwd+2,pts,
 						node.numbers,mar,add,offset,direction,setEnv,xlim,ylim,nodes,
-						tips,split.vertical,lend,asp,plot)
+						tips,split.vertical,lend,asp,plot,underscore)
 					par(fg=fg)
 				}
 				updownPhylogram(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
-				add=if(outline) TRUE else add,offset,direction,setEnv,xlim,ylim,nodes,
-				tips,split.vertical,lend,asp,plot)
+					add=if(outline) TRUE else add,offset,direction,setEnv,xlim,ylim,nodes,
+					tips,split.vertical,lend,asp,plot,underscore)
 			} else {
 				if(outline){
 					fg<-par()$fg
@@ -57,12 +57,12 @@ plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 					black[]<-fg
 					plotPhylogram(tree,colors=black,fsize,ftype,lwd=lwd+2,pts,
 						node.numbers,mar,add,offset,direction,setEnv,xlim,ylim,nodes,
-						tips,split.vertical,lend,asp,plot)
+						tips,split.vertical,lend,asp,plot,underscore)
 					par(fg=fg)
 				}
 				plotPhylogram(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
-				add=if(outline) TRUE else add,offset,direction,setEnv,xlim,ylim,nodes,
-				tips,split.vertical,lend,asp,plot)
+					add=if(outline) TRUE else add,offset,direction,setEnv,xlim,ylim,nodes,
+					tips,split.vertical,lend,asp,plot,underscore)
 			}
 		} else if(type=="fan"){
 			if(outline){
@@ -76,6 +76,18 @@ plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 			}
 			plotFan(tree,colors,fsize,ftype,lwd,mar,add=if(outline) TRUE else add,part,
 				setEnv,xlim,ylim,tips,maxY,lend,plot,offset)
+		} else if(type=="arc"){
+			if(outline){
+				fg<-par()$fg
+				par(fg="transparent")
+				black<-colors
+				black[]<-fg
+				arcPhylogram(tree,colors=black,fsize,ftype,lwd=lwd+2,mar,add,part,setEnv,
+					xlim,ylim,tips,maxY,lend,plot,offset,arc_height)
+				par(fg=fg)
+			}
+			arcPhylogram(tree,colors,fsize,ftype,lwd,mar,add=if(outline) TRUE else add,part,
+				setEnv,xlim,ylim,tips,maxY,lend,plot,offset,arc_height)
 		} else if(type=="cladogram"){
 			if(outline){
 				fg<-par()$fg
@@ -93,11 +105,42 @@ plotSimmap<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,
 	}
 }
 
+## this is a wrapper of plotFan
+## written by Liam J. Revell 2023
+arcPhylogram<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,
+	xlim,ylim,tips,maxY,lend,plot,offset,arc_height){
+	tree<-reorder(tree,"cladewise")
+	edge<-tree$edge
+	edge[edge>Ntip(tree)]<-edge[edge>Ntip(tree)]+1
+	edge<-rbind(Ntip(tree)+c(1,2),edge)
+	Nnode<-Nnode(tree)+1
+	edge.length<-c(arc_height*max(nodeHeights(tree)),tree$edge.length)
+	maps<-c(vector(length=1,mode="numeric"),tree$maps)
+	maps[[1]]<-setNames(edge.length[1],"NULO")
+	colors<-c(setNames("transparent","NULO"),colors)
+	object<-list(edge=edge,Nnode=Nnode,tip.label=tree$tip.label,
+		edge.length=edge.length,maps=maps)
+	class(object)<-class(tree)
+	attr(object,"map.order")<-attr(object,"map.order")
+	plotFan(object,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,
+		ylim,tips,maxY,lend,plot,offset)
+	if(setEnv){
+		PP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
+		ROOT<-PP$Ntip+1
+		PP$Nnode<-PP$Nnode-1
+		PP$edge<-PP$edge[2:nrow(PP$edge),]
+		PP$edge[PP$edge>PP$Ntip]<-PP$edge[PP$edge>PP$Ntip]-1
+		PP$xx<-PP$xx[-ROOT]
+		PP$yy<-PP$yy[-ROOT]
+		assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
+	}
+}
+
 ## function to plot simmap tree in type "phylogram"
-## written by Liam J. Revell 2011-2017
+## written by Liam J. Revell 2011-2023
 updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	add,offset,direction,setEnv,xlim,ylim,placement,tips,split.vertical,lend,
-	asp,plot){
+	asp,plot,underscore){
 	if(split.vertical&&!setEnv){
 		cat("split.vertical requires setEnv=TRUE. Setting split.vertical to FALSE.\n")
 		spit.vertical<-FALSE
@@ -116,7 +159,7 @@ updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	if(is.null(tips)) Y[cw$edge[cw$edge[,2]<=n,2]]<-1:n
 	else Y[cw$edge[cw$edge[,2]<=n,2]]<-if(is.null(names(tips))) 
 		tips[sapply(1:Ntip(cw),function(x,y) which(y==x),y=cw$edge[cw$edge[,2]<=n,2])]
-		else tips[gsub(" ","_",cw$tip.label)]
+		else if(!underscore) tips[gsub(" ","_",cw$tip.label)]
 	# get Y coordinates of the nodes
 	nodes<-unique(pw$edge[,1])
 	for(i in 1:m){
@@ -167,7 +210,7 @@ updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 			for(i in 1:m) lines(Y[cw$edge[which(cw$edge[,1]==nodes[i]),2]],
 				H[which(cw$edge[,1]==nodes[i]),1],
 				col=colors[names(cw$maps[[match(nodes[i],
-				cw$edge[,1])]])[1]],lwd=lwd)
+				cw$edge[,1])]])[1]],lwd=lwd,lend=lend)
 		}
 		for(i in 1:nrow(cw$edge)){
 			x<-H[i,1]
@@ -223,7 +266,7 @@ updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 			font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=offset,
 			x.lim=xlim,y.lim=ylim,
 			direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
-			edge=cw$edge,xx=Y[,1],yy=sapply(1:(Ntip(cw)+cw$Nnode),
+			edge=tree$edge,xx=Y[,1],yy=sapply(1:(Ntip(cw)+cw$Nnode),
 			function(x,y,z) y[match(x,z)],y=H,z=cw$edge))
 		assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 	}
@@ -231,10 +274,10 @@ updownPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 }
 
 # function to plot simmap tree in type "phylogram"
-# written by Liam J. Revell 2011-2017
+# written by Liam J. Revell 2011-2023
 plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	add,offset,direction,setEnv,xlim,ylim,placement,tips,split.vertical,lend,
-	asp,plot){
+	asp,plot,underscore){
 	if(split.vertical&&!setEnv){
 		cat("split.vertical requires setEnv=TRUE. Setting split.vertical to FALSE.\n")
 		spit.vertical<-FALSE
@@ -253,7 +296,7 @@ plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 	if(is.null(tips)) Y[cw$edge[cw$edge[,2]<=n,2]]<-1:n
 	else Y[cw$edge[cw$edge[,2]<=n,2]]<-if(is.null(names(tips))) 
 		tips[sapply(1:Ntip(cw),function(x,y) which(y==x),y=cw$edge[cw$edge[,2]<=n,2])]
-		else tips[gsub(" ","_",cw$tip.label)]
+		else if(!underscore) tips[gsub(" ","_",cw$tip.label)]
 	# get Y coordinates of the nodes
 	nodes<-unique(pw$edge[,1])
 	for(i in 1:m){
@@ -303,7 +346,7 @@ plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 		if(!split.vertical){
 			for(i in 1:m) lines(H[which(cw$edge[,1]==nodes[i]),1],
 				Y[cw$edge[which(cw$edge[,1]==nodes[i]),2]],col=colors[names(cw$maps[[match(nodes[i],
-				cw$edge[,1])]])[1]],lwd=lwd)
+				cw$edge[,1])]])[1]],lwd=lwd,lend=lend)
 		}
 		for(i in 1:nrow(cw$edge)){
 			x<-H[i,1]
@@ -350,7 +393,7 @@ plotPhylogram<-function(tree,colors,fsize,ftype,lwd,pts,node.numbers,mar,
 			font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=offset,
 			x.lim=xlim,y.lim=ylim,
 			direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
-			edge=cw$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
+			edge=tree$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
 			function(x,y,z) y[match(x,z)],y=H,z=cw$edge),yy=Y[,1])
 		assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 	}
@@ -452,7 +495,7 @@ plotFan<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips
 			show.tip.label=if(ftype) TRUE else FALSE,show.node.label=FALSE,
 			font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=offset,
 			x.lim=xlim,y.lim=ylim,direction="rightwards",tip.color="black",
-			Ntip=Ntip(cw),Nnode=cw$Nnode,edge=cw$edge,
+			Ntip=Ntip(cw),Nnode=cw$Nnode,edge=tree$edge,
 			xx=c(x[sapply(1:n,function(x,y) which(x==y)[1],y=cw$edge[,2]),2],x[1,1],
 			if(m>1) x[sapply(2:m+n,function(x,y) which(x==y)[1],y=cw$edge[,2]),2] else c()),
 			yy=c(y[sapply(1:n,function(x,y) which(x==y)[1],y=cw$edge[,2]),2],y[1,1],
@@ -462,10 +505,10 @@ plotFan<-function(tree,colors,fsize,ftype,lwd,mar,add,part,setEnv,xlim,ylim,tips
 }
 
 ## internal function for slanted cladogram
-## written by Liam J. Revell 2017
+## written by Liam J. Revell 2017-2023
 plotCladogram<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,mar=NULL,
 	add=FALSE,offset=NULL,direction="rightwards",xlim=NULL,ylim=NULL,
-	nodes="intermediate",tips=NULL,lend=2,asp=NA,plot=TRUE){
+	nodes="intermediate",tips=NULL,lend=2,asp=NA,plot=TRUE,underscore=FALSE){
 	placement<-nodes
 	# set offset fudge (empirically determined)
 	offsetFudge<-1.37
@@ -481,7 +524,7 @@ plotCladogram<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,mar=NULL,
 	if(is.null(tips)) Y[cw$edge[cw$edge[,2]<=n,2]]<-1:n
 	else Y[cw$edge[cw$edge[,2]<=n,2]]<-if(is.null(names(tips))) 
 		tips[sapply(1:Ntip(cw),function(x,y) which(y==x),y=cw$edge[cw$edge[,2]<=n,2])]
-		else tips[gsub(" ","_",cw$tip.label)]
+		else if(!underscore) tips[gsub(" ","_",cw$tip.label)]
 	# get Y coordinates of the nodes
 	nodes<-unique(pw$edge[,1])
 	for(i in 1:m){
@@ -558,15 +601,17 @@ plotCladogram<-function(tree,colors=NULL,fsize=1.0,ftype="reg",lwd=2,mar=NULL,
 		font=ftype,cex=fsize,adj=0,srt=0,no.margin=FALSE,label.offset=offset,
 		x.lim=xlim,y.lim=ylim,
 		direction=direction,tip.color="black",Ntip=Ntip(cw),Nnode=cw$Nnode,
-		edge=cw$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
+		edge=tree$edge,xx=sapply(1:(Ntip(cw)+cw$Nnode),
 		function(x,y,z) y[match(x,z)],y=H,z=cw$edge),yy=Y[,1])
 	assign("last_plot.phylo",PP,envir=.PlotPhyloEnv)
 }
 
 
 ## adds legend to an open stochastic map style plot
-## written by Liam J. Revell 2013, 2016, 2017
+## written by Liam J. Revell 2013, 2016, 2017, 2023
 add.simmap.legend<-function(leg=NULL,colors,prompt=TRUE,vertical=TRUE,...){
+	if(hasArg(border)) border<-list(...)$border
+	else border<-par()$fg
 	if(hasArg(shape)) shape<-list(...)$shape
 	else shape<-"square"
 	if(prompt){
@@ -596,14 +641,15 @@ add.simmap.legend<-function(leg=NULL,colors,prompt=TRUE,vertical=TRUE,...){
 		y<-rep(y+w/2,length(x))
 		text(x,y,leg,pos=4,cex=fsize/par()$cex)
 	}
-	if(shape=="square") symbols(x,y,squares=rep(w,length(x)),bg=colors,add=TRUE,inches=FALSE)
+	if(shape=="square") symbols(x,y,squares=rep(w,length(x)),bg=colors,add=TRUE,inches=FALSE,
+		fg=border)
 	else if(shape=="circle") nulo<-mapply(draw.circle,x=x,y=y,col=colors,
-		MoreArgs=list(nv=200,radius=w/2))
+		MoreArgs=list(nv=200,radius=w/2,border=border))
 	else stop(paste("shape=\"",shape,"\" is not a recognized option.",sep=""))
 }
 
 # function plots a tree; in the new version this is just a wrapper for plotSimmap
-# written by Liam Revell 2012-2017
+# written by Liam Revell 2012-2017, 2023
 plotTree<-function(tree,...){
 	if(hasArg(color)) color<-list(...)$color
 	else color<-NULL
@@ -630,7 +676,7 @@ plotTree<-function(tree,...){
 	if(hasArg(setEnv)) setEnv<-list(...)$setEnv
 	else setEnv<-TRUE
 	if(hasArg(part)) part<-list(...)$part
-	else part<-1.0
+	else part<-if(type=="arc") 0.5 else 1
 	if(hasArg(xlim)) xlim<-list(...)$xlim
 	else xlim<-NULL
 	if(hasArg(ylim)) ylim<-list(...)$ylim
@@ -649,13 +695,18 @@ plotTree<-function(tree,...){
 	else asp<-NA
 	if(hasArg(plot)) plot<-list(...)$plot
 	else plot<-TRUE
+	if(hasArg(underscore)) underscore<-list(...)$underscore
+	else underscore<-FALSE
+	if(hasArg(arc_height)) arc_height<-list(...)$arc_height
+	else arc_height<-2
 	if(inherits(tree,"multiPhylo")){
 		par(ask=TRUE)
 		if(!is.null(color)) names(color)<-"1"
 		for(i in 1:length(tree)) plotTree(tree[[i]],color=color,fsize=fsize,ftype=ftype,
 			lwd=lwd,pts=pts,node.numbers=node.numbers,mar=mar,add=add,offset=offset,
 			direction=direction,type=type,setEnv=setEnv,part=part,xlim=xlim,ylim=ylim,
-			nodes=nodes,tips=tips,maxY=maxY,hold=hold,lend=lend,asp=asp,plot=plot)
+			nodes=nodes,tips=tips,maxY=maxY,hold=hold,lend=lend,asp=asp,plot=plot,
+			underscore=underscore,arc_height=arc_height)
 	} else {
 		if(is.null(tree$edge.length)) tree<-compute.brlen(tree)
 		tree$maps<-as.list(tree$edge.length)
@@ -664,7 +715,7 @@ plotTree<-function(tree,...){
 		plotSimmap(tree,colors=color,fsize=fsize,ftype=ftype,lwd=lwd,pts=pts,
 			node.numbers=node.numbers,mar=mar,add=add,offset=offset,direction=direction,
 			type=type,setEnv=setEnv,part=part,xlim=xlim,ylim=ylim,nodes=nodes,tips=tips,maxY=maxY,
-			hold=hold,lend=lend,asp=asp,plot=plot)
+			hold=hold,lend=lend,asp=asp,plot=plot,underscore=underscore,arc_height=arc_height)
 	}
 }
 
