@@ -1,5 +1,306 @@
 ## some utility functions
-## written by Liam J. Revell 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022
+## written by Liam J. Revell 2011, 2012, 2013, 2014, 2015, 2016, 2017, 
+## 2018, 2019, 2020, 2021, 2022, 2023
+
+## function forces a tree to be ultrametric using two different methods
+## written by Liam J. Revell 2017, 2021, 2022, 2023
+
+force.ultrametric<-function(tree,method=c("nnls","extend"),...){
+	if(hasArg(message)) message<-list(...)$message
+	else message<-TRUE
+	if(message){
+		cat("***************************************************************\n")
+		cat("*                          Note:                              *\n")
+		cat("*    force.ultrametric does not include a formal method to    *\n")
+		cat("*    ultrametricize a tree & should only be used to coerce    *\n")
+		cat("*   a phylogeny that fails is.ultramtric due to rounding --   *\n")
+		cat("*    not as a substitute for formal rate-smoothing methods.   *\n")
+		cat("***************************************************************\n")
+	}
+	method<-method[1]
+	if(method=="nnls") tree<-nnls.tree(cophenetic(tree),tree,
+		method="ultrametric",rooted=is.rooted(tree),trace=0)
+	else if(method=="extend"){
+		h<-diag(vcv(tree))
+		d<-max(h)-h
+		ii<-sapply(1:Ntip(tree),function(x,y) which(y==x),
+			y=tree$edge[,2])
+		tree$edge.length[ii]<-tree$edge.length[ii]+d
+	} else 
+		cat("method not recognized: returning input tree\n\n")
+	tree
+}
+
+## c "combine" method for "simmap" and "multiSimmap" object classes
+
+## adapted from c.phylo in ape (Paradis & Schliep 2019)
+c.simmap<-function(...,recursive=TRUE){
+	obj<-list(...)
+	classes<-lapply(obj,class)
+	isphylo<-sapply(classes,function(x) "simmap" %in% x)
+	if(all(isphylo)){
+		class(obj)<-c("multiSimmap","multiPhylo")
+		return(obj)
+	}
+	if(!recursive) return(obj)
+	ismulti<-sapply(classes, function(x) "multiSimmap" %in% x)
+	if(all(isphylo|ismulti)){
+		result<-list()
+		j<-1
+		for(i in 1:length(isphylo)){
+			if(isphylo[i]){ 
+				result[[j]]<-obj[[i]]
+				j<-j+1
+			} else {
+				n<-length(obj[[i]])
+				result[0:(n-1)+j]<-.uncompressTipLabel(obj[[i]])
+				j<-j+n
+			}
+		}
+		class(result)<-c("multiSimmap","multiPhylo")
+		obj<-result
+	} else {
+		msg<-paste("some objects not of class \"simmap\" or",
+			"\"multiSimmap\": argument recursive=TRUE ignored")
+		warning(msg)
+	}
+	return(obj)
+}
+
+## adapted from c.multiPhylo in ape (Paradis & Schliep 2019)
+c.multiSimmap<-function(...,recursive=TRUE){
+	obj<-list(...)
+	if(!recursive) return(obj)
+	classes<-lapply(obj,class)
+	isphylo<-sapply(classes,function(x) "simmap" %in% x)
+	ismulti<-sapply(classes, function(x) "multiSimmap" %in% x)
+	if(all(isphylo|ismulti)){
+		result<-list()
+		j<-1
+		for(i in 1:length(isphylo)){
+			if(isphylo[i]){ 
+				result[[j]]<-obj[[i]]
+				j<-j+1
+			} else {
+				n<-length(obj[[i]])
+				result[0:(n-1)+j]<-.uncompressTipLabel(obj[[i]])
+				j<-j+n
+			}
+		}
+		class(result)<-c("multiSimmap","multiPhylo")
+		obj<-result
+	} else {
+		msg<-paste("some objects not of class \"simmap\" or",
+			"\"multiSimmap\": argument recursive=TRUE ignored")
+		warning(msg)
+	}
+	return(obj)
+}
+
+## function to summarize the results of stochastic mapping
+## written by Liam J. Revell 2013, 2014, 2015, 2021, 2022, 2023
+
+describe.simmap<-function(tree,...){
+	if(hasArg(plot)) plot<-list(...)$plot
+	else plot<-FALSE
+	if(hasArg(check.equal)) check.equal<-list(...)$check.equal
+	else check.equal<-FALSE
+	if(hasArg(message)) message<-list(...)$message
+	else message<-FALSE
+	if(hasArg(ref.tree)) ref.tree<-list(...)$ref.tree
+	else ref.tree<-NULL
+	if(inherits(tree,"multiPhylo")){
+		if(check.equal){
+			TT<-sapply(tree,function(x,y) sapply(y,all.equal.phylo,x),y=tree)
+			check<-all(TT)
+			if(!check) cat("Note: Some trees are not equal.\nA \"reference\" tree will be computed if none was provided.\n\n")
+		} else check<-TRUE
+		if(is.null(ref.tree)&&check){ 
+			YY<-getStates(tree,"both")
+			states<-sort(unique(as.vector(YY)))
+			ZZ<-t(apply(YY,1,function(x,levels,Nsim) summary(factor(x,levels))/Nsim,
+				levels=states,Nsim=length(tree)))
+		} else {
+			YY<-getStates(tree)
+			states<-sort(unique(as.vector(YY)))
+			if(is.null(ref.tree)){
+				cat("No reference tree provided & some trees are unequal.\nComputing majority-rule consensus tree.\n")
+				ref.tree<-consensus(tree,p=0.5)
+			}
+			YYp<-matrix(NA,ref.tree$Nnode,length(tree),dimnames=list(1:ref.tree$Nnode+Ntip(ref.tree),
+				NULL))
+			for(i in 1:length(tree)){
+				M<-matchNodes(ref.tree,tree[[i]])
+				jj<-sapply(M[,2],function(x,y) if(x%in%y) which(as.numeric(y)==x) else NA,
+					y=as.numeric(rownames(YY)))
+				YYp[,i]<-YY[jj,i]
+			}
+			ZZ<-t(apply(YYp,1,function(x,levels) summary(factor(x[!is.na(x)],
+				levels))/sum(!is.na(x)),levels=states))
+		}
+		XX<-countSimmap(tree,states,FALSE)
+		XX<-XX[,-(which(as.vector(diag(-1,length(states)))==-1)+1)]
+		AA<-lapply(unclass(tree),function(x) setNames(
+			c(colSums(x$mapped.edge),sum(x$edge.length)),
+			c(colnames(x$mapped.edge),"total")))
+		foo<-function(x,n){
+			y<-setNames(rep(0,length(n)),n)
+			y[names(x)]<-x
+			y
+		}
+		AA<-t(sapply(AA,foo,n=c(states,"total")))
+		BB<-getStates(tree,type="tips")
+		CC<-t(apply(BB,1,function(x,levels,Nsim) summary(factor(x,levels))/Nsim,levels=states,
+			Nsim=length(tree)))
+		x<-list(count=XX,times=AA,ace=ZZ,tips=CC,tree=tree,ref.tree=if(!is.null(ref.tree)) 
+			ref.tree else NULL)
+		class(x)<-"describe.simmap"
+	} else if(inherits(tree,"phylo")){
+		XX<-countSimmap(tree,message=FALSE)
+		YY<-getStates(tree)
+		states<-sort(unique(YY))
+		AA<-setNames(c(colSums(tree$mapped.edge),sum(tree$edge.length)),
+			c(colnames(tree$mapped.edge),"total"))
+		AA<-rbind(AA,AA/AA[length(AA)]); rownames(AA)<-c("raw","prop")
+		x<-list(N=XX$N,Tr=XX$Tr,times=AA,states=YY,tree=tree)
+		class(x)<-"describe.simmap"
+	}
+	if(message) print(x)
+	if(plot) plot(x)
+	x
+}
+
+# function works like extract.clade in ape but will preserve a discrete character mapping
+# written by Liam J. Revell 2013, 2023
+extract.clade.simmap<-function(tree,node){
+	if(!inherits(tree,"simmap")) stop("tree should be an object of class \"simmap\".")
+	x<-getDescendants(tree,node)
+	x<-x[x<=Ntip(tree)]
+	drop.tip.simmap(tree,tree$tip.label[-x])
+}
+
+## function to add an arrow pointing to a tip or node in the tree
+## written by Liam J. Revell 2014, 2017, 2020, 2023
+
+add.arrow<-function(tree=NULL,tip,...){
+	if(length(tip)>1){ 
+		object<-sapply(tip,add.arrow,tree=tree,...)
+		invisible(object)
+	} else {
+		lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
+		asp<-if(lastPP$type=="fan") 1 else (par()$usr[4]-par()$usr[3])/(par()$usr[2]-
+			par()$usr[1])
+		if(!is.null(tree)){
+			if(inherits(tree,"contMap")) tree<-tree$tree
+			else if(inherits(tree,"densityMap")) tree<-tree$tree
+		}
+		if(is.numeric(tip)){
+			ii<-tip
+			if(!is.null(tree)&&ii<=Ntip(tree)) tip<-tree$tip.label[ii]
+			else tip<-""
+		} else if(is.character(tip)&&!is.null(tree)) ii<-which(tree$tip.label==tip)
+		if(hasArg(offset)) offset<-list(...)$offset
+		else offset<-lastPP$label.offset
+		strw<-lastPP$cex*(strwidth(tip)+offset*mean(strwidth(c(LETTERS,letters))))
+		if(lastPP$direction%in%c("upwards","downwards")) 
+			strw<-strw*asp*par()$pin[1]/par()$pin[2]
+		if(hasArg(arrl)) arrl<-list(...)$arrl
+		else { 
+			if(lastPP$type=="fan") arrl<-0.3*max(lastPP$xx)
+			else if(lastPP$type=="phylogram") arrl<-0.15*max(lastPP$xx)
+		}
+		if(hasArg(hedl)) hedl<-list(...)$hedl
+		else hedl<-arrl/3
+		if(hasArg(angle)) angle<-list(...)$angle
+		else angle<-45
+		arra<-angle*pi/180
+		if(hasArg(col)) col<-list(...)$col
+		else col<-"black"
+		if(hasArg(lwd)) lwd<-list(...)$lwd	
+		else lwd<-2
+		if(lastPP$type=="fan") theta<-atan2(lastPP$yy[ii],lastPP$xx[ii])
+		else if(lastPP$type=="phylogram"){
+			if(lastPP$direction=="rightwards") theta<-0
+			else if(lastPP$direction=="upwards") theta<-pi/2 
+			else if(lastPP$direction=="leftwards") theta<-pi
+			else if(lastPP$direction=="downwards") theta<-3*pi/2
+		}
+		segments(x0=lastPP$xx[ii]+cos(theta)*(strw+arrl),
+			y0=lastPP$yy[ii]+sin(theta)*(strw+arrl),
+			x1=lastPP$xx[ii]+cos(theta)*strw,
+			y1=lastPP$yy[ii]+sin(theta)*strw,
+			col=col,lwd=lwd,lend="round")
+		segments(x0=lastPP$xx[ii]+cos(theta)*strw+cos(theta+arra/2)*hedl,
+			y0=lastPP$yy[ii]+sin(theta)*strw+sin(theta+arra/2)*hedl*asp,
+			x1=lastPP$xx[ii]+cos(theta)*strw,
+			y1=lastPP$yy[ii]+sin(theta)*strw,
+			col=col,lwd=lwd,lend="round")
+		segments(x0=lastPP$xx[ii]+cos(theta)*strw+cos(theta-arra/2)*hedl,
+			y0=lastPP$yy[ii]+sin(theta)*strw+sin(theta-arra/2)*hedl*asp,
+			x1=lastPP$xx[ii]+cos(theta)*strw,
+			y1=lastPP$yy[ii]+sin(theta)*strw,
+			col=col,lwd=lwd,lend="round")
+		invisible(list(x0=lastPP$xx[ii]+cos(theta)*(strw+arrl),
+			y0=lastPP$yy[ii]+sin(theta)*(strw+arrl),
+			x1=lastPP$xx[ii]+cos(theta)*strw,
+			y1=lastPP$yy[ii]+sin(theta)*strw))
+	}
+}
+
+## function to rescale simmap style trees
+## written by Liam J. Revell 2012, 2013, 2014, 2015, 2017, 2023
+
+## S3 Ntip etc. methods for "contMap" object class
+Ntip.contMap<-function(phy) Ntip(phy$tree)
+Nnode.contMap<-function(phy,...) Nnode(phy$tree)
+Nedge.contMap<-function(phy) Nedge(phy$tree)
+
+## S3 Ntip etc. methods for "densityMap" object class
+Ntip.densityMap<-function(phy) Ntip(phy$tree)
+Nnode.densityMap<-function(phy,...) Nnode(phy$tree)
+Nedge.densityMap<-function(phy) Nedge(phy$tree)
+
+rescale<-function(x,...) UseMethod("rescale")
+
+rescale.default<-function(x,...){
+	warning(paste(
+		"rescale does not know how to handle objects of class ",class(x),".\n",
+		"if",class(x),"= \"phylo\" load geiger package to rescale.\n"))
+}
+
+rescale.simmap<-function(x, model="depth", ...) rescaleSimmap(x,...)
+
+rescale.multiSimmap<-function(x, model="depth", ...) rescaleSimmap(x,...)
+
+rescaleSimmap<-function(tree,...){
+	if(inherits(tree,"multiSimmap")){
+		cls<-class(tree)
+		trees<-unclass(tree)
+		trees<-lapply(trees,rescaleSimmap,...)
+		class(trees)<-cls
+		return(trees)
+	} else if(inherits(tree,"simmap")){
+		if(hasArg(lambda)) lambda<-list(...)$lambda
+		else lambda<-1
+		if(hasArg(totalDepth)) depth<-totalDepth<-list(...)$totalDepth
+		else if(hasArg(depth)) depth<-totalDepth<-list(...)$depth
+		else depth<-totalDepth<-max(nodeHeights(tree))
+		if(lambda!=1){
+			e<-lambdaTree(tree,lambda)$edge.length/tree$edge.length
+			tree$edge.length<-tree$edge.length*e
+			tree$maps<-mapply(function(x,y) x*y,tree$maps,e)
+			tree$mapped.edge<-tree$mapped.edge*matrix(rep(e,ncol(tree$mapped.edge)),length(e),ncol(tree$mapped.edge))
+		}
+		if(depth!=max(nodeHeights(tree))){
+			h<-max(nodeHeights(tree))
+			s<-depth/h
+			tree$edge.length<-tree$edge.length*s
+			tree$maps<-lapply(tree$maps,"*",s)
+			tree$mapped.edge<-tree$mapped.edge*s
+		}
+		return(tree)
+	} else message("tree should be an object of class \"simmap\" or \"multiSimmap\"")
+}
 
 ## function to get states at internal nodes from simmap style trees
 ## written by Liam J. Revell 2013, 2014, 2015, 2021
@@ -102,8 +403,8 @@ expand.clade<-function(tree,node,factor=5){
 	cw<-reorder(tree)
 	tips<-setNames(rep(1,Ntip(tree)),cw$tip.label)
 	get.tips<-function(node,tree){
-    		dd<-getDescendants(tree,node)
-    		tree$tip.label[dd[dd<=Ntip(tree)]]
+				dd<-getDescendants(tree,node)
+				tree$tip.label[dd[dd<=Ntip(tree)]]
 	}
 	desc<-unlist(lapply(node,get.tips,tree=cw))
 	for(i in 2:Ntip(cw)){
@@ -123,7 +424,7 @@ expand.clade<-function(tree,node,factor=5){
 print.expand.clade<-function(x,...){
 	cat("An object of class \"expand.clade\" consisting of:\n")
 	cat(paste("(1) A phylogenetic tree (x$tree) with",Ntip(x$tree),
-		"tips and\n   ",x$tree$Nnode,"internal nodes.\n"))
+		"tips and\n	 ",x$tree$Nnode,"internal nodes.\n"))
 	cat("(2) A vector (x$tips) containing the desired tip-spacing.\n\n")
 }
 
@@ -307,9 +608,9 @@ get.asp<-function(){
 	asp
 }
 
-round.polygon<-function(x,y,col="transparent"){
+# round.polygon<-function(x,y,col="transparent"){
 	## just space holding for now	
-}
+# }
 
 ## draw a box around a clade
 ## written by Liam J. Revell 2017
@@ -377,35 +678,6 @@ linklabels<-function(text,tips,link.type=c("bent","curved","straight"),
 		segments(xpos,ypos,xmax,ylab,lty=lty,col=col)
 }
 
-## function forces a tree to be ultrametric using two different methods
-## written by Liam J. Revell 2017, 2021
-
-force.ultrametric<-function(tree,method=c("nnls","extend"),...){
-	if(hasArg(message)) message<-list(...)$message
-	else message<-TRUE
-	if(message){
-		cat("***************************************************************\n")
-		cat("*                          Note:                              *\n")
-		cat("*    force.ultrametric does not include a formal method to    *\n")
-		cat("*    ultrametricize a tree & should only be used to coerce    *\n")
-		cat("*   a phylogeny that fails is.ultramtric due to rounding --   *\n")
-		cat("*    not as a substitute for formal rate-smoothing methods.   *\n")
-		cat("***************************************************************\n")
-	}
-	method<-method[1]
-	if(method=="nnls") tree<-nnls.tree(cophenetic(tree),tree,
-		rooted=TRUE,trace=0)
-	else if(method=="extend"){
-		h<-diag(vcv(tree))
-		d<-max(h)-h
-		ii<-sapply(1:Ntip(tree),function(x,y) which(y==x),
-			y=tree$edge[,2])
-		tree$edge.length[ii]<-tree$edge.length[ii]+d
-	} else 
-		cat("method not recognized: returning input tree\n\n")
-	tree
-}
-
 ## function to create curved clade labels for a fan tree
 ## written by Liam J. Revell 2017, 2022
 
@@ -548,7 +820,7 @@ print.aic.w<-function(x,...){
 }
 
 ## function to compute all paths towards the tips from a node
-## written by  Liam J. Revell
+## written by	Liam J. Revell
 node.paths<-function(tree,node){
 	d<-Descendants(tree,node,"children")
 	paths<-as.list(d)
@@ -893,12 +1165,12 @@ markChanges<-function(tree,colors=NULL,cex=1,lwd=2,plot=TRUE){
 }
 
 ## function to label clades
-## written by Liam J. Revell 2014, 2015
+## written by Liam J. Revell 2014, 2015, 2022
 cladelabels<-function(tree=NULL,text,node,offset=NULL,wing.length=NULL,cex=1,
 	orientation="vertical"){
 	lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
 	if(is.null(tree)){
-		wing.length<-1
+		if(is.null(wing.length)) wing.length<-1
 		if(is.null(offset)) offset<-8
 		tree<-list(edge=lastPP$edge,
 			tip.label=1:lastPP$Ntip,
@@ -910,11 +1182,11 @@ cladelabels<-function(tree=NULL,text,node,offset=NULL,wing.length=NULL,cex=1,
 	if(is.null(offset)) offset<-0.5
 	xx<-mapply(labelSubTree,node,text,
 		MoreArgs=list(tree=tree,pp=lastPP,offset=offset,wl=wing.length,cex=cex,
-		orientation=orientation))
+		orientation=orientation),SIMPLIFY=FALSE)
 }
 
 ## internal function used by cladelabels
-## written by Liam J. Revell 2014, 2015
+## written by Liam J. Revell 2014, 2015, 2022
 labelSubTree<-function(tree,nn,label,pp,offset,wl,cex,orientation){
 	if(is.null(wl)) wl<-1
 	tree<-reorder(tree)
@@ -928,14 +1200,16 @@ labelSubTree<-function(tree,nn,label,pp,offset,wl,cex,orientation){
 		nodeHeights(tree)[which(tree$edge[,2]==x),2],
 		tree=tree))+sw+offset*cw
 	y<-range(pp$yy[tips])
-	lines(c(h,h),y+ec*c(-sh,sh))
+	lines(c(h,h),y+ec*c(-sh,sh),col=par()$fg)
 	lines(c(h-wl*cw,h),
-		c(y[1]-ec*sh,y[1]-ec*sh))
+		c(y[1]-ec*sh,y[1]-ec*sh),col=par()$fg)
 	lines(c(h-wl*cw,h),
-		c(y[2]+ec*sh,y[2]+ec*sh))
+		c(y[2]+ec*sh,y[2]+ec*sh),col=par()$fg)
 	text(h+cw,mean(y),
 		label,srt=if(orientation=="horizontal") 0 else 90,
-		adj=if(orientation=="horizontal") 0 else 0.5,cex=cex)
+		adj=if(orientation=="horizontal") 0 else 0.5,cex=cex,
+		col=par()$col.lab)
+	list(xx=c(h,h),yy=y+ec*c(-sh,sh))
 }
 
 ## get all the extant/extinct tip names
@@ -992,7 +1266,7 @@ drop.clade<-function(tree,tip){
 	chk<-tree$tip.label[desc[desc<=Ntip(tree)]]
 	if(!setequal(tip,chk)){
 		cat("Caution: Species in tip do not form a monophyletic clade.\n")
-		cat("         Pruning all tips descended from ancestor.\n\n")
+		cat("				 Pruning all tips descended from ancestor.\n\n")
 		tip<-chk
 	}
 	## step 2, find all edges in the clade & set them to zero length
@@ -1027,7 +1301,7 @@ reroot<-function(tree,node.number,position=NULL,interactive=FALSE,...){
 		else position<-tree$edge.length[which(tree$edge[,2]==node.number)]
 	} else {
 		if(node.number==(Ntip(tree)+1))
-			cat("      A value of position != 0 has been reset to zero.\n")
+			cat("			A value of position != 0 has been reset to zero.\n")
 	}
 	if(hasArg(edgelabel)) edgelabel<-list(...)$edgelabel
 	else edgelabel<-FALSE
@@ -1050,67 +1324,6 @@ reroot<-function(tree,node.number,position=NULL,interactive=FALSE,...){
 	obj
 }
 
-## function to add an arrow pointing to a tip or node in the tree
-## written by Liam J. Revell 2014, 2017, 2020
-
-add.arrow<-function(tree=NULL,tip,...){
-	if(length(tip)>1){ 
-		object<-sapply(tip,add.arrow,tree=tree,...)
-		invisible(object)
-	} else {
-		lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
-		if(!is.null(tree)){
-			if(inherits(tree,"contMap")) tree<-tree$tree
-			else if(inherits(tree,"densityMap")) tree<-tree$tree
-		}
-		if(is.numeric(tip)){
-			ii<-tip
-			if(!is.null(tree)&&ii<=Ntip(tree)) tip<-tree$tip.label[ii]
-			else tip<-""
-		} else if(is.character(tip)&&!is.null(tree)) ii<-which(tree$tip.label==tip)
-		if(hasArg(offset)) offset<-list(...)$offset
-		else offset<-1
-		strw<-lastPP$cex*(strwidth(tip)+offset*mean(strwidth(c(LETTERS,letters))))
-		if(hasArg(arrl)) arrl<-list(...)$arrl
-		else { 
-			if(lastPP$type=="fan") arrl<-0.3*max(lastPP$xx)
-			else if(lastPP$type=="phylogram") arrl<-0.15*max(lastPP$xx)
-		}
-		if(hasArg(hedl)) hedl<-list(...)$hedl
-		else hedl<-arrl/3
-		if(hasArg(angle)) angle<-list(...)$angle
-		else angle<-45
-		arra<-angle*pi/180
-		asp<-if(lastPP$type=="fan") 1 else (par()$usr[4]-par()$usr[3])/(par()$usr[2]-
-			par()$usr[1])
-		if(hasArg(col)) col<-list(...)$col
-		else col<-"black"
-		if(hasArg(lwd)) lwd<-list(...)$lwd	
-		else lwd<-2
-		if(lastPP$type=="fan") theta<-atan2(lastPP$yy[ii],lastPP$xx[ii])
-		else if(lastPP$type=="phylogram") theta<-0	
-		segments(x0=lastPP$xx[ii]+cos(theta)*(strw+arrl),
-			y0=lastPP$yy[ii]+sin(theta)*(strw+arrl),
-			x1=lastPP$xx[ii]+cos(theta)*strw,
-			y1=lastPP$yy[ii]+sin(theta)*strw,
-			col=col,lwd=lwd,lend="round")
-		segments(x0=lastPP$xx[ii]+cos(theta)*strw+cos(theta+arra/2)*hedl,
-			y0=lastPP$yy[ii]+sin(theta)*strw+sin(theta+arra/2)*hedl*asp,
-			x1=lastPP$xx[ii]+cos(theta)*strw,
-			y1=lastPP$yy[ii]+sin(theta)*strw,
-			col=col,lwd=lwd,lend="round")
-		segments(x0=lastPP$xx[ii]+cos(theta)*strw+cos(theta-arra/2)*hedl,
-			y0=lastPP$yy[ii]+sin(theta)*strw+sin(theta-arra/2)*hedl*asp,
-			x1=lastPP$xx[ii]+cos(theta)*strw,
-			y1=lastPP$yy[ii]+sin(theta)*strw,
-			col=col,lwd=lwd,lend="round")
-		invisible(list(x0=lastPP$xx[ii]+cos(theta)*(strw+arrl),
-			y0=lastPP$yy[ii]+sin(theta)*(strw+arrl),
-			x1=lastPP$xx[ii]+cos(theta)*strw,
-			y1=lastPP$yy[ii]+sin(theta)*strw))
-		}
-}
-
 ## function to ladderize phylogeny with mapped discrete character
 ## written by Liam J. Revell 2014, 2015, 2019
 
@@ -1173,65 +1386,35 @@ rep.multiPhylo<-function(x,...){
 	obj
 }
 
-## function to rescale simmap style trees
-## written by Liam J. Revell 2012, 2013, 2014, 2015, 2017
-rescaleSimmap<-function(tree,...){
-	if(inherits(tree,"multiPhylo")){
-		cls<-class(tree)
-		trees<-unclass(tree)
-		trees<-lapply(trees,rescaleSimmap,...)
-		class(trees)<-cls
-		return(trees)
-	} else if(inherits(tree,"phylo")){
-		if(hasArg(lambda)) lambda<-list(...)$lambda
-		else lambda<-1
-		if(hasArg(totalDepth)) depth<-totalDepth<-list(...)$totalDepth
-		else if(hasArg(depth)) depth<-totalDepth<-list(...)$depth
-		else depth<-totalDepth<-max(nodeHeights(tree))
-		if(lambda!=1){
-			e<-lambdaTree(tree,lambda)$edge.length/tree$edge.length
-			tree$edge.length<-tree$edge.length*e
-			tree$maps<-mapply(function(x,y) x*y,tree$maps,e)
-			tree$mapped.edge<-tree$mapped.edge*matrix(rep(e,ncol(tree$mapped.edge)),length(e),ncol(tree$mapped.edge))
-		}
-		if(depth!=max(nodeHeights(tree))){
-			h<-max(nodeHeights(tree))
-			s<-depth/h
-			tree$edge.length<-tree$edge.length*s
-			tree$maps<-lapply(tree$maps,"*",s)
-			tree$mapped.edge<-tree$mapped.edge*s
-		}
-		return(tree)
-	} else message("tree should be an object of class \"phylo\" or \"multiPhylo\"")
-}
-
 ## function to drop one or more tips from a tree but retain all ancestral nodes as singletons
-## written by Liam J. Revell 2014, 2015
-drop.tip.singleton<-function(tree,tip){
-	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
-	N<-Ntip(tree)
+## written by Liam J. Revell 2014, 2015, 2023
+drop.tip.singleton<-function(phy,tip,...){
+	if(!inherits(phy,"singleton")&&!inherits(phy,"phylo")) 
+		stop("phy should be an object of class \"singleton\".")
+	N<-Ntip(phy)
 	m<-length(tip)
-	ii<-sapply(tip,function(x,y) which(y==x),y=tree$tip.label)
-	tree$tip.label<-tree$tip.label[-ii]
-	ii<-sapply(ii,function(x,y) which(y==x),y=tree$edge[,2])
-	tree$edge<-tree$edge[-ii,]
-	tree$edge.length<-tree$edge.length[-ii]
-	tree$edge[tree$edge<=N]<-as.integer(rank(tree$edge[tree$edge<=N]))
-	tree$edge[tree$edge>N]<-tree$edge[tree$edge>N]-m
+	ii<-sapply(tip,function(x,y) which(y==x),y=phy$tip.label)
+	phy$tip.label<-phy$tip.label[-ii]
+	ii<-sapply(ii,function(x,y) which(y==x),y=phy$edge[,2])
+	phy$edge<-phy$edge[-ii,]
+	phy$edge.length<-phy$edge.length[-ii]
+	phy$edge[phy$edge<=N]<-as.integer(rank(phy$edge[phy$edge<=N]))
+	phy$edge[phy$edge>N]<-phy$edge[phy$edge>N]-m
 	N<-N-m
-	if(any(sapply(tree$edge[tree$edge[,2]>N,2],"%in%",tree$edge[,1])==FALSE)) internal<-TRUE
+	if(any(sapply(phy$edge[phy$edge[,2]>N,2],"%in%",phy$edge[,1])==FALSE)) internal<-TRUE
 	else internal<-FALSE
 	while(internal){
-		ii<-which(sapply(tree$edge[,2],"%in%",c(1:N,tree$edge[,1]))==FALSE)[1]
-		nn<-tree$edge[ii,2]
-		tree$edge<-tree$edge[-ii,]
-		tree$edge.length<-tree$edge.length[-ii]
-		tree$edge[tree$edge>nn]<-tree$edge[tree$edge>nn]-1
-		tree$Nnode<-tree$Nnode-length(ii)
-		if(any(sapply(tree$edge[tree$edge[,2]>N,2],"%in%",tree$edge[,1])==FALSE)) internal<-TRUE
+		ii<-which(sapply(phy$edge[,2],"%in%",c(1:N,phy$edge[,1]))==FALSE)[1]
+		nn<-phy$edge[ii,2]
+		phy$edge<-phy$edge[-ii,]
+		phy$edge.length<-phy$edge.length[-ii]
+		phy$edge[phy$edge>nn]<-phy$edge[phy$edge>nn]-1
+		phy$Nnode<-phy$Nnode-length(ii)
+		if(any(sapply(phy$edge[phy$edge[,2]>N,2],"%in%",phy$edge[,1])==FALSE)) internal<-TRUE
 		else internal<-FALSE
 	}
-	tree
+	class(phy)<-union("singleton",class(phy))
+	phy
 }
 
 ## S3 print method for object of class 'describe.simmap'
@@ -1292,62 +1475,6 @@ plot.describe.simmap<-function(x,...){
 	}
 }
 
-
-## function to summarize the results of stochastic mapping
-## written by Liam J. Revell 2013, 2014, 2015, 2021
-describe.simmap<-function(tree,...){
-	if(hasArg(plot)) plot<-list(...)$plot
-	else plot<-FALSE
-	if(hasArg(check.equal)) check.equal<-list(...)$check.equal
-	else check.equal<-FALSE
-	if(hasArg(message)) message<-list(...)$message
-	else message<-FALSE
-	if(hasArg(ref.tree)) ref.tree<-list(...)$ref.tree
-	else ref.tree<-NULL
-	if(inherits(tree,"multiPhylo")){
-		if(check.equal){
-			TT<-sapply(tree,function(x,y) sapply(y,all.equal.phylo,x),y=tree)
-			check<-all(TT)
-			if(!check) cat("Note: Some trees are not equal.\nA \"reference\" tree will be computed if none was provided.\n\n")
-		} else check<-TRUE
-		YY<-getStates(tree,"both")
-		states<-sort(unique(as.vector(YY)))
-		if(is.null(ref.tree)&&check) ZZ<-t(apply(YY,1,function(x,levels,Nsim) summary(factor(x,levels))/Nsim,levels=states,Nsim=length(tree)))
-		else {
-			if(is.null(ref.tree)){
-				cat("No reference tree provided & some trees are unequal.\nComputing majority-rule consensus tree.\n")
-				ref.tree<-consensus(tree,p=0.5)
-			}
-			YYp<-matrix(NA,ref.tree$Nnode,length(tree),dimnames=list(1:ref.tree$Nnode+Ntip(ref.tree),NULL))
-			for(i in 1:length(tree)){
-				M<-matchNodes(ref.tree,tree[[i]])
-				jj<-sapply(M[,2],function(x,y) if(x%in%y) which(as.numeric(y)==x) else NA,y=as.numeric(rownames(YY)))
-				YYp[,i]<-YY[jj,i]
-			}
-			ZZ<-t(apply(YYp,1,function(x,levels) summary(factor(x[!is.na(x)],levels))/sum(!is.na(x)),levels=states))
-		}
-		XX<-countSimmap(tree,states,FALSE)
-		XX<-XX[,-(which(as.vector(diag(-1,length(states)))==-1)+1)]
-		AA<-t(sapply(unclass(tree),function(x) c(colSums(x$mapped.edge),sum(x$edge.length))))
-		colnames(AA)[ncol(AA)]<-"total"
-		BB<-getStates(tree,type="tips")
-		CC<-t(apply(BB,1,function(x,levels,Nsim) summary(factor(x,levels))/Nsim,levels=states,Nsim=length(tree)))
-		x<-list(count=XX,times=AA,ace=ZZ,tips=CC,tree=tree,ref.tree=if(!is.null(ref.tree)) ref.tree else NULL)
-		class(x)<-"describe.simmap"
-	} else if(inherits(tree,"phylo")){
-		XX<-countSimmap(tree,message=FALSE)
-		YY<-getStates(tree)
-		states<-sort(unique(YY))
-		AA<-setNames(c(colSums(tree$mapped.edge),sum(tree$edge.length)),c(colnames(tree$mapped.edge),"total"))
-		AA<-rbind(AA,AA/AA[length(AA)]); rownames(AA)<-c("raw","prop")
-		x<-list(N=XX$N,Tr=XX$Tr,times=AA,states=YY,tree=tree)
-		class(x)<-"describe.simmap"
-	}
-	if(message) print(x)
-	if(plot) plot(x)
-	x
-}
-
 ## function finds the height of a given node
 ## written by Liam Revell 2014, 2015, 2016
 nodeheight<-function(tree,node,...){
@@ -1415,7 +1542,8 @@ getAncestors<-function(tree,node,type=c("all","parent")){
 
 ## function for midpoint rooting
 ## written by Liam J. Revell 2014
-midpoint.root<-function(tree){
+## (being deprecated out in 2023)
+midpoint_root<-function(tree){
 	D<-cophenetic(tree)
 	dd<-max(D)
 	ii<-which(D==dd)[1]
@@ -1433,6 +1561,9 @@ midpoint.root<-function(tree){
 	tree
 }
 
+midpoint.root<-function(tree,node.labels="support",...) 
+	phangorn::midpoint(tree,node.labels="support",...)
+
 # function computes phylogenetic variance-covariance matrix, including for internal nodes
 # written by Liam J. Revell 2011, 2013, 2014, 2015
 vcvPhylo<-function(tree,anc.nodes=TRUE,...){
@@ -1492,7 +1623,7 @@ lambdaTree<-function(tree,lambda){
 	H1<-nodeHeights(tree)
 	tree$edge.length[ii]<-lambda*tree$edge.length[ii]
 	H2<-nodeHeights(tree)
-	tree$edge.length[-ii]<-tree$edge.length[-ii]+     H1[-ii,2]-H2[-ii,2]
+	tree$edge.length[-ii]<-tree$edge.length[-ii]+		 H1[-ii,2]-H2[-ii,2]
 	tree
 }
 
@@ -1541,20 +1672,20 @@ di2multi.simmap<-function(phy,...){
 # written by Liam J. Revell 2011, 2012, 2013, 2015, 2016
 # modified by Klaus Schliep 2017
 nodeHeights<-function(tree,...){
-    if(hasArg(root.edge)) root.edge<-list(...)$root.edge
-    else root.edge<-FALSE
-    if(root.edge) ROOT<-if(!is.null(tree$root.edge)) tree$root.edge else 0
-    else ROOT<-0 
-    nHeight <- function(tree){
-        tree <- reorder(tree)
-        edge <- tree$edge
-        el <- tree$edge.length
-        res <- numeric(max(tree$edge))
-        for(i in seq_len(nrow(edge))) res[edge[i,2]] <- res[edge[i,1]] + el[i] 
-        res
-    }
-    nh <- nHeight(tree)
-    return(matrix(nh[tree$edge], ncol=2L)+ROOT)
+		if(hasArg(root.edge)) root.edge<-list(...)$root.edge
+		else root.edge<-FALSE
+		if(root.edge) ROOT<-if(!is.null(tree$root.edge)) tree$root.edge else 0
+		else ROOT<-0 
+		nHeight <- function(tree){
+				tree <- reorder(tree)
+				edge <- tree$edge
+				el <- tree$edge.length
+				res <- numeric(max(tree$edge))
+				for(i in seq_len(nrow(edge))) res[edge[i,2]] <- res[edge[i,1]] + el[i] 
+				res
+		}
+		nh <- nHeight(tree)
+		return(matrix(nh[tree$edge], ncol=2L)+ROOT)
 }
 
 ## function drops all the leaves from the tree & collapses singleton nodes
@@ -1700,8 +1831,8 @@ sampleFrom<-function(xbar=0,xvar=1,n=1,randn=NULL,type="norm"){
 
 	for(i in 1:length(xbar)){
 		y<-rnorm(n=n[i],mean=xbar[i],sd=sqrt(xvar[i]))
-   		names(y)<-rep(names(xbar)[i],length(y))
-   		x<-c(x,y)
+	 		names(y)<-rep(names(xbar)[i],length(y))
+	 		x<-c(x,y)
 	}
 	return(x)
 }
@@ -1788,7 +1919,7 @@ findMRCA<-function(tree,tips=NULL,type=c("node","height")){
 			X<-apply(X,c(1,2),function(x,y,z) y[which(z==x)[1]],y=H,z=tree$edge)
 		}
 		return(X)
-    } else {
+		} else {
 		node<-getMRCA(tree,tips)
 		if (type == "node") return(node)
 		else if(type=="height") return(nodeheight(tree,node))
